package com.elab.test.guava;

import com.google.common.eventbus.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * EventBus 消息总线
 *
 * @author liukaixiong
 * @Email liukx@elab-plus.com
 * @date 2021/12/3 - 13:47
 */
public class EventBusSample {
    private static Logger logger = LoggerFactory.getLogger(EventBusSample.class);

    @Subscribe
    public void A(AModel model) {
        logger.info("A : " + model.toString());
    }

    @Subscribe
    public void A1(AModel model) {
        logger.info("A1 : " + model.toString());
    }

    @Subscribe
    @AllowConcurrentEvents
    public void B(BModel model) {
        logger.info("B : " + model.toString());
    }

    @Subscribe
    public void C(CModel model) {
        logger.info("C : " + model.toString());
    }

    /**
     * 这里代表死信事件，代表内部没有找到对应的订阅者
     *
     * @param event
     */
    @Subscribe
    public void handle(DeadEvent event) {
        //获取事件源
        logger.info(event.getSource().toString());
        //获取事件
        logger.info(event.getEvent() + toString());
    }

    public static class AModel {
        private String username;

        public String getUsername() {
            return username;
        }

        public void setUsername(String username) {
            this.username = username;
        }
    }

    public static class BModel {
        private String username;

        public String getUsername() {
            return username;
        }

        public void setUsername(String username) {
            this.username = username;
        }
    }

    public static class CModel {
        private String username;

        public String getUsername() {
            return username;
        }

        public void setUsername(String username) {
            this.username = username;
        }
    }

    public static class DModel extends AModel {

    }

    public static void main(String[] args) {
        EventBus eventBus = new EventBus((exception, context) -> {
            System.out.println(exception.getMessage());
        });
        logger.info("主线程开始A");
        eventBus.register(new EventBusSample());
        eventBus.post(new DModel());
        eventBus.post(new BModel());
        logger.info("主线程结束A");

        ExecutorService executorService = Executors.newFixedThreadPool(5);
        // 异步处理
        AsyncEventBus asyncEventBus = new AsyncEventBus(executorService);
        logger.info("主线程开始B");
        asyncEventBus.register(new EventBusSample());
        asyncEventBus.post(new DModel());
        asyncEventBus.post(new BModel());
        logger.info("主线程结束B");
    }

}
